﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;


namespace Pic113Application
{
    public partial class FormDisplay : Form
    {
        // Create the serial port with basic settings
        private SerialPort port;

        public FormDisplay()
        {
            InitializeComponent();
            port = null;

            checkPortAvailable();   //scan for the available port on the system
        }

        private void checkPortAvailable()
        {
            string[] ArrayComPortsNames = null;
            int index = -1;
            string ComPortName = null;

            comboBox_ComPort.Items.Clear();

            ArrayComPortsNames = SerialPort.GetPortNames();
            do
            {
                index += 1;
                comboBox_ComPort.Items.Add(ArrayComPortsNames[index]);
            }
            while (!((ArrayComPortsNames[index] == ComPortName) ||
                                (index == ArrayComPortsNames.GetUpperBound(0))));
        }

        private void updateInputStatus()
        {
            //send polling messages to get the status of the current input
            string cmd_pollInput1 = "\x03IP01\r\n";
            string cmd_pollInput2 = "\x03IP02\r\n";
            port.Write(cmd_pollInput1);
            System.Threading.Thread.Sleep(100);
            port.Write(cmd_pollInput2);
        }

        private void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            Console.Write("Data Received->\"");
            // Show all the incoming data in the port's buffer
            string str = port.ReadExisting();
            Console.Write(str);
            Console.WriteLine("\"");

            string cmd_in1On = "\x03IP0101\r\n";
            string cmd_in1Off = "\x03IP0100\r\n";
            string cmd_in2On = "\x03IP0201\r\n";
            string cmd_in2Off = "\x03IP0200\r\n";

            if(string.Compare(str, cmd_in1On) == 0)
            {
                Console.WriteLine("1 On");
                input1But.BackColor = Color.GreenYellow;
            }
            else if (string.Compare(str, cmd_in1Off) == 0)
            {
                Console.WriteLine("1 Off");
                input1But.BackColor = Color.Silver;
            }
            else if (string.Compare(str, cmd_in2On) == 0)
            {
                Console.WriteLine("2 On");
                input2But.BackColor = Color.GreenYellow;
            }
            else if (string.Compare(str, cmd_in2Off) == 0)
            {
                Console.WriteLine("2 Off");
                input2But.BackColor = Color.Silver;
            }
        }

        private void buttonP3on_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port3 <On>");
            string cmd = "\x03OP0301\r\n";
            port.Write(cmd);
        }

        private void buttonP3off_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port3 <Off>");
            string cmd = "\x03OP0300\r\n";
            port.Write(cmd);
        }

        private void buttonP3toggle_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port3 <Toggle>");
            string cmd = "\x03OP03\r\n";
            port.Write(cmd);
        }

        private void buttonP3pulsepos_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port3 <Pulse+>");
            string cmd = "\x03O103\r\n";
            port.Write(cmd);
        }

        private void buttonP3pulseneg_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port3 <Pulse->");
            string cmd = "\x03O203\r\n";
            port.Write(cmd);
        }

        private void buttonP4on_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port4 <On>");
            string cmd = "\x03OP0401\r\n";
            port.Write(cmd);
        }

        private void buttonP4off_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port4 <Off>");
            string cmd = "\x03OP0400\r\n";
            port.Write(cmd);
        }

        private void buttonP4toggle_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port4 <Toggle>");
            string cmd = "\x03OP04\r\n";
            port.Write(cmd);
        }

        private void buttonP4pulsepos_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port4 <Pulse+>");
            string cmd = "\x03O104\r\n";
            port.Write(cmd);
        }

        private void buttonP4pulseneg_Click(object sender, EventArgs e)
        {
            if (!isConnected()) return;     //check if COM port is connected
            Console.WriteLine("Clicked Port4 <Pulse->");
            string cmd = "\x03O204\r\n";
            port.Write(cmd);
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            string portName = comboBox_ComPort.Text;

            if (port == null)    //if port is not opened yet
            {
                try
                {
                    Console.WriteLine("Connecting COM PORT <" + portName + ">");
                    port = new SerialPort(portName, 9600, Parity.None, 8, StopBits.One);

                    // Attach a method to be called when there
                    // is data waiting in the port's buffer
                    port.DataReceived += new SerialDataReceivedEventHandler(port_DataReceived);

                    // Begin communications
                    port.Open();
                    connectBut.BackColor = Color.GreenYellow;
                    connectBut.Text = "Disconnect";

                    //update the status of the input, once connected
                    updateInputStatus();    //update the input status on the display
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: " + ex.ToString(), "ERROR");
                }
            }
            else                //if port is already open
            {
                if (port.IsOpen) //if port is already open, disconnect it
                {
                    Console.WriteLine("Disconnecting COM PORT <" + portName + ">");
                    port.Close();
                    port = null;
                    connectBut.BackColor = Color.LightGray;
                    connectBut.Text = "Connect";

                    //disable input status indicator
                    input1But.BackColor = Color.Silver;
                    input2But.BackColor = Color.Silver;
                    return;
                }
            }
        }

        private Boolean isConnected()
        {
            if (port == null);
            else if (port.IsOpen)
                return (true);
            //if com port is null or not open yet
            MessageBox.Show("COM port not opened yet.", "ERROR");
            return (false);
        }

        private void scan_Click(object sender, EventArgs e)
        {
            checkPortAvailable();
        }
    }
}
